/*
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __LAUNCHPAD_COMMON_H__
#define __LAUNCHPAD_COMMON_H__

#define _GNU_SOURCE

#include <unistd.h>
#include <ctype.h>
#include <dlog.h>
#include <bundle_internal.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdbool.h>
#include <tzplatform_config.h>

#ifdef LAUNCHPAD_LOG
#undef LOG_TAG
#define LOG_TAG "LAUNCHPAD"
#endif

#define MAX_PENDING_CONNECTIONS 10
#define MAX_LOCAL_BUFSZ 128
#define AUL_SOCK_MAXBUFF 131071
#define LOADER_ARG_LEN 1024

#define PAD_CMD_LAUNCH		0
#define PAD_CMD_VISIBILITY	10
#define PAD_CMD_ADD_LOADER	11
#define PAD_CMD_REMOVE_LOADER	12
#define PAD_CMD_MAKE_DEFAULT_SLOTS	13
#define PAD_CMD_DEMAND		14
#define PAD_CMD_PING		15
#define PAD_CMD_UPDATE_APP_TYPE	16

#define LAUNCHPAD_LAUNCH_SIGNAL 83
#define LAUNCHPAD_DEAD_SIGNAL 61
#define APP_STARTUP_SIGNAL 89
#define LAUNCHPAD_CHILD_PROCESS 130

#define PAD_LOADER_ID_STATIC	0
#define PAD_LOADER_ID_DIRECT	1
#define PAD_LOADER_ID_DYNAMIC_BASE	10

#define _E(fmt, arg...) LOGE(fmt, ##arg)
#define _D(fmt, arg...) LOGD(fmt, ##arg)
#define _W(fmt, arg...) LOGW(fmt, ##arg)
#define _I(fmt, arg...) LOGI(fmt, ##arg)

#define FREE_AND_NULL(x) do {		\
	if (x) {			\
		free(x);		\
		x = NULL;		\
	}				\
} while (0)

#define ARRAY_SIZE(x) ((sizeof(x)) / sizeof(x[0]))
#define GLOBAL_USER tzplatform_getuid(TZ_SYS_GLOBALAPP_USER)

typedef struct _app_pkt_t {
	int cmd;
	int len;
	int opt;
	unsigned char data[1];
} app_pkt_t;

typedef struct {
	char *appid;
	char *app_path;
	char *original_app_path;
	char *pkg_type;
	char *app_type;
	char *hwacc;
	char *taskmanage;
	char *pkgid;
	char *comp_type;
	char *internal_pool;
	char *root_path;
	char *loader_name;
	bool global;
} appinfo_t;

void _modify_bundle(bundle *kb, int caller_pid, appinfo_t *menu_info, int cmd);

int _send_message_to_logger(const char *tag, const char *format, ...);
int _send_cmd_to_amd(int cmd);
int _create_server_sock(const char *name);
app_pkt_t *_recv_pkt_raw(int fd);
app_pkt_t *_accept_recv_pkt_raw(int fd, int *clifd, struct ucred *cr);
int _send_pkt_raw(int client_fd, app_pkt_t *pkt);
int  _connect_to_launchpad(int type, int id);
int _set_sock_option(int fd, int cli);
void _set_env(appinfo_t *menu_info, bundle *kb);
char **_create_argc_argv(bundle *kb, int *margc);
char *_get_libdir(const char *path);
int _delete_sock_path(int pid, uid_t uid);

appinfo_t *_appinfo_create(bundle *kb);
void _appinfo_free(appinfo_t *menu_info);
char *_appinfo_get_app_path(appinfo_t *menu_info);
int _proc_get_attr_by_pid(int pid, char *buf, int size);
int _close_all_fds(void);
void _get_cpu_idle(unsigned long long *total, unsigned long long *idle);
int _setup_stdio(const char *ident);
int _set_priority(int prio);
int _wait_tep_mount(bundle *b);
int _prepare_app_socket(void);
int _enable_external_pkg(bundle *b, const char *pkgid, uid_t pkg_uid);
int _verify_proc_caps(void);
int _prepare_id_file(void);
void _print_hwc_log(const char *format, ...);

#endif /* __LAUNCHPAD_COMMON_H__ */

