/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <stdbool.h>
#include <glib.h>
#include <bundle.h>

#define DEFAULT_CPU_THRESHOLD_MAX		90
#define DEFAULT_CPU_THRESHOLD_MIN		40

enum loader_method_e {
	METHOD_TIMEOUT = 0x0001,
	METHOD_VISIBILITY = 0x0002,
	METHOD_DEMAND = 0x0004,
	METHOD_REQUEST = 0x0010,
	METHOD_AVAILABLE_MEMORY = 0x0020,
	METHOD_TTL = 0x0040,
	METHOD_OUT_OF_MEMORY = 0x0100,
	METHOD_INSTALL = 0x0200,
};

typedef struct _loader_info {
	int type;
	char *name;
	char *exe;
	GList *app_types;
	int detection_method;
	int timeout_val;
	char *hw_acc;
	GList *alternative_loaders;
	bundle *extra;
	int cpu_threshold_max;
	int cpu_threshold_min;
	bool on_boot;
	bool global;
	bool app_exists;
	int activation_method;
	int deactivation_method;
	unsigned int ttl;
	bool is_hydra;
} loader_info_t;

typedef void (*loader_info_foreach_cb)(loader_info_t *info, void *data);

GList *_loader_info_load(const char *path);
void _loader_info_dispose(GList *info);
int _loader_info_find_type(GList *info, const char *app_type, bool hwacc);
int _loader_info_find_type_by_loader_name(GList *info, const char *loader_name);
int *_loader_get_alternative_types(GList *info, int type, int *len);
int _loader_info_foreach(GList *info, loader_info_foreach_cb callback,
		void *data);
bool _loader_info_exist_app_type(loader_info_t *info, const char *app_type);
